from functools import wraps

from flask import g, abort

from myslx.utils import has_roles, redirect_to_login


def login_required(f):

    @wraps(f)
    def decorated_function(*args, **kwargs):
        if g.user is None:
            return redirect_to_login()

        return f(*args, **kwargs)

    return decorated_function


def requires_roles(*roles):

    def wrapper(f):

        @wraps(f)
        def decorated_function(*args, **kwargs):
            # this implies the user is authenticated
            if g.user is None:
                return redirect_to_login()

            if not has_roles(g.user['authorities'], roles):
                abort(403)
            return f(*args, **kwargs)

        return decorated_function

    return wrapper



import os

PORTAL_DOMAIN = os.environ.get('DOMAIN', 'my.solarlux.com')
OAUTH_CLIENT_ID = os.environ.get('OAUTH_CLIENT_ID', 'localdev1')
OAUTH_CLIENT_SECRET = os.environ.get(
    'OAUTH_CLIENT_SECRET', 'fDtxU18ltTRk59PLu5VRoh670QoRkaym'
)  # '6Htrqkzel6uqeuHZZrjA5Q2EXka4kw10')
_default_public_key = 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAiahLhO5+1bWfqSQhdCR7RJLpveU0OICX5rPb4N5AMx6uGhwCgO9U44kkg1Z/K+atcokYJUW72pij0TjFpz9XOAIR1BbpzpsSE/ixe1cRx2kDG1ABiCvWBsldnuuW/KhZi9FtQGnVBysKiXT/0Y87MtJo63ZQwKOK9DSdQWiz/zWgij9oR4Q2Cl3iBrQ3h8HMYEswhKrZfO+GEz8BKUgLeUMr6TJtJQX85Z+ulWcTsxbAZZPzAiNlnBbbd8MV3rimEcfE/0vbRoFXZ/yfQ8PlD09MdrCXjCGnr1WKrKkse9HgTDjnBiewMZiHswYfHHiEqSCw4o3U2XtjGGXWSZiv5wIDAQAB'
OAUTH_PUBLIC_KEY = """-----BEGIN PUBLIC KEY-----
%s
-----END PUBLIC KEY-----""" % os.environ.get(
    'OAUTH_PUBLIC_KEY', _default_public_key
)
SCRIPT_NAME = os.environ.get('SCRIPT_NAME', '')



from werkzeug.utils import redirect
from flask import url_for, request

from myslx import config


def has_roles(user_roles, required_roles):
    # Falls die Liste der required_roles leer oder None ist True zurück geben
    if not required_roles:
        return True

    for required_role in required_roles:
        # Überprüft ob die Aktuelle rolle in den User Rollen vorhanden ist
        if required_role in user_roles:
            # Falls auch nur eine der Rollen vorhanden ist Return True
            return True

    # Falls keine der Rollen vorhanden ist Return false
    return False


def redirect_to_login():
    uri = url_for('myslx.callback', next=request.url, _external=True)
    return redirect(
        f"https://{config.PORTAL_DOMAIN}/auth/oauth/authorize?client_id={config.OAUTH_CLIENT_ID}&response_type=code&redirect_uri={uri}"
    )



import jwt
import requests
from flask import Blueprint, make_response, redirect, abort, request, g, current_app, render_template, url_for, Response
from jwt import InvalidTokenError
from requests.auth import HTTPBasicAuth

from myslx import config
from myslx.utils import has_roles

myslx = Blueprint('myslx', __name__, template_folder='templates')


@myslx.before_app_request
def before_request():
    g.user = None
    access_token = request.cookies.get('access_token')
    if not access_token:
        return

    try:
        data = jwt.decode(
            access_token, config.OAUTH_PUBLIC_KEY, algorithms=['RS256']
        )
    except InvalidTokenError as e:
        current_app.logger.warn('authentication failed, token invalid: %s' % e)
        return

    g.user = data


@myslx.route('/accounts/callback')
def callback():
    code = request.args.get('code')
    if not code:
        abort(403)
    uri = url_for('myslx.callback', next=request.args.get('next'), _external=True)
    res = requests.post(
        f"https://{config.PORTAL_DOMAIN}/auth/oauth/token?grant_type=authorization_code&code={code}&redirect_uri={uri}",
        auth=HTTPBasicAuth(config.OAUTH_CLIENT_ID, config.OAUTH_CLIENT_SECRET),
        headers={'Content-Type': 'application/x-www-form-urlencoded'},
        timeout=10,
    )
    if res.status_code != 200:
        current_app.logger.warn(
            'authentication failed, auth svc responsed with %s',
            res.status_code,
        )
        abort(403)
    res_data = res.json()
    access_token = res_data['access_token']
    try:
        data = jwt.decode(
            access_token, config.OAUTH_PUBLIC_KEY, algorithms=['RS256']
        )
        current_app.logger.info('%s logged in successfully', data['email'])
    except InvalidTokenError as e:
        current_app.logger.warn('authentication failed, token invalid: %s' % e)
        abort(403)
    resp = redirect(request.args.get('next'))
    for c in ['access_token', 'refresh_token']:
        resp.set_cookie(c, res_data[c])
    resp.set_cookie('expiration_time', str(res_data['expires_in']))
    return resp


@myslx.route('/logout')
def logout():
    resp = make_response(render_template('logged_out.html'))
    for c in ['access_token', 'refresh_token', 'expiration_time']:
        resp.set_cookie(c, '')
    return resp


@myslx.route('/ping')
def test_connection():
    return Response(status=200)



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()
# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))
setup(
    name='myslx-flask',
    version='0.7.0',
    packages=find_packages('.'),
    include_package_data=True,
    license='BSD License',
    description='Integration components for mySolarlux for Flask based apps.',
    long_description=README,
    url='https://git.solarlux.com/sl-it-dev/slx8174_myslx-flask/',
    author='Enno Lohmeier',
    author_email='e.lohmeier@solarlux.de',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    keywords='django auth account oauth',
    install_requires=['Flask', 'cryptography', 'PyJWT', 'requests'],
)
